!building a realistic water surface by finite differencing
! of water waves propagation
program buildreal3d
use sep
implicit none
  integer  :: n1, n2, n3, j3 , choise
  real, parameter :: PI=3.14, &!cos(-1.), &
                      v2=.2**2., base=10
  real, dimension(:,:), pointer :: Pt, Ptp, Ptm, tmp !P-now, P+1, P-1
  !counters and temprorary variables
  integer :: tt, ii, jj
  call sep_init();
  call from_param("n1", n1, 300);
  call from_param("n2", n2, 300);
  call from_param("n3", n3, 100);
  call from_param("j3", j3, 1);
  call from_param("choise", choise, 3);
  call to_history("label1","x(1unit)");
  call to_history("label2","y(1unit)");
  call to_history("n1", n1);
  call to_history("n2", n2);
  call to_history("n3", n3);
  ! Allocating space for the variables with additional space for boundary
  ! conditions
  allocate(Pt(0:n1+1,0:n2+1), Ptp(0:n1+1,0:n2+1), Ptm(0:n1+1,0:n2+1));
  !allocate(Pt(n1,n2), Ptp(n1,n2), Ptm(n1,n2));
  !finite differencing of the 2d-wave equation
  !initial conditions and boundary condtions where at boundaries p=0
  Pt=0;
  Ptm=0;
  do tt=0, n3*j3-1
    ptp=0;!assuming that boudry condtions are p=0
    do ii=1,n1
      do jj=1,n2
        ptp(ii,jj)=v2*(source(tt,ii,jj)+Pt(ii+1,jj)+Pt(ii,jj+1) &
                         -1./v2*Ptm(ii,jj)+Pt(ii-1,jj)+Pt(ii,jj-1) &
                         +2.*Pt(ii,jj)*(1./v2-2.));
      enddo
    enddo
    tmp=>Ptm;
    Ptm=>Pt;
    Pt=>Ptp;
    Ptp=>tmp;
    if(mod(tt,j3)==0) then 
      write (0,*) 'Writng frame #', tt
      call sep_write(Ptp(1:n1,1:n2)+base);
    endif
  enddo
  !end of wave equation finite differencing
  nullify(tmp, Pt, Ptm, Ptp)
contains 
  real function source(t,x,y)
  implicit none
    integer:: t, x, y, x0, y0, x1, y1
    real :: ts, cov 
    source=0;
    cov=3
    x0=n1*.5;
    y0=n2*.5;
    x1=n1*.3;
    y1=n2*.4;
    !source= sin(tt*.1)*exp(-(((t-ts)/(cov*10))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
    select case(choise)

      case(1) ! for nice short display
        ts = cov*4; ! start
        source= sin(tt*.2)*exp(-(((t-ts)/(cov*30))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
        source= source+cos(tt*.2)*exp(-(((t-ts)/(cov*30))**2.+((x-x1)/cov)**2.+((y-y1)/cov)**2.));

      case(2) ! very long wavelength for 100 * 100 grid
        ts = cov*16; ! start
        source= sin(tt*.06)*exp(-(((t-ts)/(cov*100))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
        source= source-.8*cos(tt*.07)*exp(-(((t-ts)/(cov*100))**2.+((x-x1)/cov)**2.+((y-y1)/cov)**2.));

      case(3) ! very long wavelength for 200 * 200 grid
        ts = cov*100; ! start
        source= sin(tt*.03)*exp(-(((t-ts)/(cov*200))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
        source= source-.6*cos(tt*.04)*exp(-(((t-ts)/(cov*200))**2.+((x-x1)/cov)**2.+((y-y1)/cov)**2.));
        source= source-.4*cos(tt*.05)*exp(-(((t-ts)/(cov*200))**2.+((x-x1*2)/cov)**2.+((y-y1*2.)/cov)**2.));
      case(4) ! very long wavelength for 1024* 1024 grid
        cov=10
        ts = 1; ! start
        source= sin(tt*.03)*exp(-(((t-ts)/(cov*200))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
        source= source-.6*cos(tt*.04)*exp(-(((t-ts)/(cov*200))**2.+((x-x1)/cov)**2.+((y-y1)/cov)**2.));
        source= source-.4*cos(tt*.05)*exp(-(((t-ts)/(cov*200))**2.+((x-x1*2)/cov)**2.+((y-y1*2.)/cov)**2.));
      case(5) ! very long wavelength for 1024* 1024 grid
        x0=n1*.4;
        y0=n2*.7;
        x1=n1*.3;
        y1=n2*.4;
        cov=5
        ts = 1; ! start
        source= sin(tt*.03)*exp(-(((t-ts)/(cov*100))**2.+((x-x0)/cov)**2.+((y-y0)/cov)**2.));
        source= source-.6*cos(tt*.04)*exp(-(((t-ts)/(cov*100))**2.+((x-x1)/cov)**2.+((y-y1)/cov)**2.));
        source= source-.7*cos(tt*.03)*exp(-(((t-ts)/(cov*100))**2.+((x-x1*2)/cov)**2.+((y-y1*2.)/cov)**2.));
    end select
    !source= (t-ts)/(td**2.)*exp(-(((t-ts)/n3)**2.));
  end function
end program

