% Forward modeling of 3D exploding water surface 
%parameters
ESIZE=500; % width of experiment
DMAX=50; % maximum depth of projection
%end of parameters

figure('visible','off')
 % this is here for no real reason (Just a hackers trick for bringing up the plot window before the animation starts)
disp(datestr(now)); % this is to print out the current time on command window
t=20;
ttl=strcat('Depth Slice:',int2str(t));
% building the surface function
sSurface = zeros(ESIZE,ESIZE);
for i=1:ESIZE
    for j=1:ESIZE
        switch(5)%Choose a surface function of the ones below or make your own
            case 0 % nothing flat surface                
            case 1 % Gaussian anamoly
                sSurface(i,j) =-1/sqrt(2*pi)*50*exp(-((4*t-i)^2+(4*t-j)^2)/30^2);
            case 2 %dens sinsoidal function
                sSurface(i,j) = sin(12/(10+0.02*i)*180/pi+j*.4-.4*sqrt(4*t))*1.5;
            case 3 % loose sinsidal function
                sSurface(i,j) = 3*sin(12/(10+0.02*i)*180/pi+j*.05-.4*sqrt(4*t));
            case 4 % The realistic one 3 sinsoidal functions superposed
               %firts wave
               x=50;y=40;z=sqrt((i-x)^2+(j-y)^2);
                if(i~=x || j~=y)
                    sSurface(i,j)=10/sqrt(z)*sin(0.12*z+5*t);
                else
                    sSurface(i,j)=sSurface(i-1,j);
                end
                %second one
                x=200;y=180;z=sqrt((i-x)^2+(j-y)^2);
                if(i~=x || j~=y)
                    sSurface(i,j)=sSurface(i,j)-cos(0.10*z)*exp(-z);
                else
                    sSurface(i,j)=sSurface(i-1,j);
                end 
                %third one
                x=10;y=200;z=sqrt((i-x)^2+(j-y)^2);
                if(i~=x || j~=y)
                    sSurface(i,j)=sSurface(i,j)-16/sqrt(5*z+10*t^2)*sin(0.15*z+pi/2*t);
                else
                    sSurface(i,j)=sSurface(i-1,j);
                end 
            case 5
                %firts wave
                x=200;y=200;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=              -9.0*exp(-z/140)*cos(0.09*z);
                x=400;y=400;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=sSurface(i,j)+6.0*exp(-z/500)*cos(0.07*z);
                %second one
                x=200;y=800;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=sSurface(i,j)-10*exp(-z/130)*cos(0.03*z);
                %third one
                x=800;y=200;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=sSurface(i,j)-15*exp(-z/150)*cos(0.05*z);
                %fourth one
                x=-200;y=200;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=sSurface(i,j)-5*exp(-z/90)*cos(0.03*z);
                %fifth one
                x=200;y=-200;z=sqrt((i-x)^2+(j-y)^2);
                sSurface(i,j)=sSurface(i,j)+20*exp(-z/150)*cos(0.08*z);
        end

    end
end
%imagesc(-sSurface);break
%end of building surface
%find the x and y-derivatives
sssdrvx=zeros(ESIZE);
sssdrvy=zeros(ESIZE);
for i=1:ESIZE
    for j=2:ESIZE-1
        sssdrvx(i,j) = (sSurface(i,j+1)-sSurface(i,j-1))/2;
    end
    sssdrvx(i,1)=sssdrvx(i,2)+(sssdrvx(i,2)-sssdrvx(i,3));
    sssdrvx(i,ESIZE)=sssdrvx(i,ESIZE-1)+(sssdrvx(i,ESIZE-1)-sssdrvx(i,ESIZE-2));
end

for i=1:ESIZE
    for j=2:ESIZE-1
        sssdrvy(j,i) = (sSurface(j+1,i)-sSurface(j-1,i))/2;
    end
    sssdrvy(1,i)=sssdrvy(2,i)+(sssdrvy(2,i)-sssdrvy(3,i));
    sssdrvy(ESIZE,i)=sssdrvy(ESIZE-1,i)+(sssdrvy(ESIZE-1,i)-sssdrvy(ESIZE-2,i));
end

% Setting up current X&Y-position of each vector
curSX=zeros(ESIZE,ESIZE);
curSY=zeros(ESIZE,ESIZE);
for i=1:ESIZE
    curSX(i,:)=1:ESIZE;
    curSY(i,:)=1:ESIZE;
end
%start building the projection matrix (the matrix holding the ray
%paths)
xmin=min(min(sSurface));
xmax=max(max(sSurface));
difference=abs(xmax-xmin);
diff32=int32(difference/2);% pad the the variations fo surface functions to the top of the matrix
Projection=zeros(diff32+DMAX, ESIZE, ESIZE,'int8'); % setup the image space
disp( 'Starting scattering')
for d=1:(diff32+DMAX) % depth from mean surface
    for vi=1:ESIZE
        for vj=1:ESIZE
            if((d-diff32)>sSurface(vi,vj))
                curSX(vi,vj)=curSX(vi,vj)-sssdrvx(vi,vj);
                curSY(vi,vj)=curSY(vi,vj)-sssdrvy(vj,vi);
                tmpX=int16(curSX(vi,vj));
                tmpY=int16(curSY(vj,vi));
                if( tmpX > 0 && tmpX < ESIZE && tmpY > 0 && tmpY<ESIZE)
                    Projection(d,tmpX, tmpY)=Projection(d,tmpX, tmpY)+1;
                end
            end
        end
    end
    disp(strcat('@depth :',int2str(d)))
end


for i=1:ESIZE
    for j=1:ESIZE
        Projection(1:int32(sSurface(i,j))+diff32+1,i,j)=10;
    end
end
%plotting

display=transpose(reshape(Projection(DMAX,:,:), ESIZE, ESIZE));
colormap gray
imagesc(display);caxis([0 9]);colorbar('XTickLabel','Ray Count','XAxisLocation','top');
set(gca,'FontName','Helvetica','FontSize',16,'Clipping','off');
%title('Binned Intensity Field','Fontsize', 12);
xlab=xlabel('x (cm)');
ylab=ylabel('y (cm)');
set(xlab,'FontName','Helvetica','FontSize',16);
set(ylab,'FontName','Helvetica','FontSize',16);
print -deps2 fig_binning3d.ps
exit;
