% Forward modeling of 2D exploding water surface 
%parameters
ESIZE=200; % width of experiment(# of vertices, notice # of surfaces/rays is ESIZE-1)
DMAX=400; % maximum depth of projection
%end of parameters
SLICE=400;%250 for precaustics

figure('visible','off')
% building the surface function
sSurface = zeros(ESIZE,1);

for i=1:ESIZE
    jj=i*2.05;
    sSurface(i) = 1.25*sin(0.01*i*pi+1.5)+1.0*sin(0.02*i*pi+1.5);
end
sSurface = sSurface-sSurface(1);

%end of building surface

%find the x-derivative
sssdrv=zeros(ESIZE-1,1);
for i=1:ESIZE-1
    sssdrv(i) = (sSurface(i+1)-sSurface(i));
end


%start building the projection matrix (the matrix holding the ray paths)
    curS=(1:(ESIZE-1))+0.5; % half way between two vertices.
    xmin=min(sSurface);
    xmax=max(sSurface);
    difference=abs(-xmin+1);% pad the the variations fo surface functions to the top of the matrix
    diff32=int32(difference);
    Projection=zeros(diff32+DMAX,ESIZE); % setup the image space
    for d=1:diff32+DMAX % d is depth from maximum of the surface function.
        for v=1:ESIZE-1
            if((d-diff32)>sSurface(v))
                curS(v)=curS(v)-sssdrv(v);
                tmpX=int32(curS(v));
                if( tmpX > 0 && tmpX < ESIZE)
                    Projection(d,tmpX)=Projection(d,tmpX)+1;
                end
            end
        end
    end
SUPERMAX=max(max(Projection)); % scaling factor
for i=1:ESIZE
    tmpX=int32(sSurface(i))+diff32;
    if(tmpX>0)
        Projection(1:tmpX,i)=-10;
        Projection(tmpX,i)=10;
    end
end

subplot(6,2,1);
plot(-sSurface,'Color',[0 0 0]);ylabel('z (cm)','Fontsize', 12);%ylim([0 max(-sSurface)])
set(gca,'YTick',[0:2:4],'YTickLabels',[4:-2:0])
intensity=Projection(SLICE,:);
subplot(6,2,[ 3 5 7 9]);
imagesc(Projection(1:SLICE+diff32,:),[0 SUPERMAX]);title('');ylabel('z (cm)','Fontsize', 12);
%set(gca,'YTick',[0:50:400],'YTickLabels',[-0:-50:-400])
subplot(6,2,11);
bar(intensity,1,'FaceColor',[0.502 0.502 0.502]);ylabel('Ray Count','Fontsize', 12); xlabel('x (cm)','Fontsize', 12);xlim([1 ESIZE]);


saveSurface=sSurface;
%Implemen the las vegas method for determining the sulution
PDEPTH=SLICE;%200;

%redistributing the surface segments to the ray projection point at DEPTH
dd=intensity;
indxr=zeros(ESIZE-1,1);
countt=0;
for i=1:ESIZE
    countt=countt+intensity(i);
end

for i=1:countt
    j=1;%1+int32(rand *(ESIZE-1));
    while(dd(j)==0)
        j=mod(j+1,ESIZE);
        if(j==0) 
            j=1;
        end
    end
    dd(j)=dd(j)-1;
    indxr(i)=j+1;
end

%reconstructing the surface
rSurface = zeros(ESIZE,1);
rSurface(1)=0;
for i=2:ESIZE
    if(indxr(i-1)<1 || indxr(i-1)>ESIZE )
        if(i<ESIZE/2)
            indxr(i-1)=0;
        else
            indxr(i-1)=ESIZE+1;
        end
    end
    SPD=rSurface(i-1)+PDEPTH;
    triangle=sqrt(SPD^2-2*(indxr(i-1)-(i+0.5)));
    rSurface(i)=(rSurface(i-1)+(-SPD+triangle));
end
    %rSurface=rSurface*1.4;
%Start building the projection
xmax=max(rSurface);
xmin=min(rSurface);
difference=abs(xmax-xmin);% pad the the variations fo surface functions to the top of the matrix
diff32=int32(difference);
Projection=zeros(diff32+PDEPTH,ESIZE); % setup the image space
for i=1:ESIZE
    tmpX=int32(rSurface(i)*2)+diff32;
    if(tmpX>0)
        Projection(tmpX,i) = 10;
    end
end



sSurface=rSurface;
%find the x-derivative
sssdrv=zeros(ESIZE-1,1);
for i=1:ESIZE-1
    sssdrv(i) = (sSurface(i+1)-sSurface(i));
end


%start building the projection matrix (the matrix holding the ray paths)
curS=(1:(ESIZE-1))+0.5; % half way between two vertices.
xmin=min(sSurface);
xmax=max(sSurface);
difference=abs(xmax-xmin);% pad the the variations fo surface functions to the top of the matrix
diff32=int32(difference);
Projection=zeros(diff32+PDEPTH,ESIZE); % setup the image space
for d=1:diff32+PDEPTH % d is depth from maximum of the surface function.
    for v=1:ESIZE-1
        if((d-diff32)>sSurface(v))
            curS(v)=curS(v)-sssdrv(v);
            tmpX=int32(curS(v));
            if( tmpX > 0 && tmpX < ESIZE)
                Projection(d,tmpX)=Projection(d,tmpX)+1;
            end
        end
    end
end
for i=1:ESIZE
    tmpX=int32(sSurface(i))+diff32;
    if(tmpX>0)
        Projection(tmpX,i)=10;
    end
end

countt1=0;
countt2=0;
for i=1:ESIZE
    countt1=countt1+intensity(i);
    countt2=countt2+Projection(PDEPTH,i);
end
sssdrv=zeros(ESIZE-1,1);
    for i=1:ESIZE-1
        sssdrv(i) = (sSurface(i+1)-sSurface(i));
    end


    %start building the projection matrix (the matrix holding the ray paths)
    curS=(1:(ESIZE-1))+0.5; % half way between two vertices.
    xmin=min(sSurface);
    xmax=max(sSurface);
    difference=abs(-xmin+1);% pad the the variations fo surface functions to the top of the matrix
    diff32=int32(difference);
    Projection=zeros(diff32+PDEPTH,ESIZE); % setup the image space
    for d=1:diff32+PDEPTH % d is depth from maximum of the surface function.
        for v=1:ESIZE-1
            if((d-diff32)>sSurface(v))
                curS(v)=curS(v)-sssdrv(v);
                tmpX=int32(curS(v));
                if( tmpX > 0 && tmpX < ESIZE)
                    Projection(d,tmpX)=Projection(d,tmpX)+1;
                end
            end
        end
    end
for i=1:ESIZE
    tmpX=int32(sSurface(i))+diff32;
    if(tmpX>0)
        Projection(1:tmpX,i)=-10;
        Projection(tmpX,i)=10;
    end
end
colormap(gray);
subplot(6,2,2);
plot(-sSurface,'Color',[0 0 0]);ylabel('z (cm)','Fontsize', 12);
set(gca,'YTick',[0:2:4],'YTickLabels',[4:-2:0])
subplot(6,2,[4 6 8 10]);
imagesc(Projection,[0 SUPERMAX]);title('');ylabel('z (cm)','Fontsize', 12);
%set(gca,'YTick',[0:50:400],'YTickLabels',[-0:-50:-400])
subplot(6,2,12);
bar(Projection(PDEPTH,:),1,'FaceColor',[0.502 0.502 0.502]);ylabel('Ray Count','Fontsize', 12); xlabel('x (cm)','Fontsize', 12);xlim([1 ESIZE]);
bestSurface=sSurface; %best of all
 brighten(.7); 
print -deps2 fig_subcaustics.ps
%exit;
