program model2d 
use sep 
use water2dlib
use waterlib
implicit none
  ! inputa
  real, dimension (:), allocatable :: surface
  real, dimension (:), pointer :: normal
  integer :: n1,n2, param_isRayMovie
  ! local variables 
  real, dimension(:,:), allocatable:: data
  integer :: ii, jj
  ! end local variables
  call sep_init();
  call from_history("n1",n2);
  call from_param("n1",n1,300);
  call from_param("raymovie",param_isRayMovie,0  );
  call to_history("n1",n1);
  call to_history("n2",n2);
  call to_history("label1","x(cm)");
  call to_history("label2","z(cm)");
  allocate(surface(n2), normal(n2-1), data(n1,n2));
  call sep_read(surface);
  normal=> getSurfaceNormal(surface);
  select case(param_isRayMovie)
    case (0)
      data=0;
      do ii=1,n1
        call makeLightScan(surface, normal, data(ii,:), real(ii));
        !call makeLight(data(ii,:), real(ii));
      enddo
      call addSurface()
      call sep_write(data);

    case (1) ! do one moving ray along the surface
      call to_history("n3",n2-1);
      do jj=1,n2-1
          data=0;
        do ii=1,n1
          call makeLightScan(surface, normal, data(ii,:), real(ii),jj);
          !call makeLight(data(ii,:), real(ii),jj);
        enddo
        call addSurface()
        call sep_write(data);
      enddo
  end select
  deallocate(surface,normal, data);
contains
  ! creates a slice through the conceptual light pattern
  ! This uses the ray theory without scanning
  subroutine makeLight(slice, depth, oneRay) 
  implicit none
    real, intent(inout), dimension(:) :: slice
    real, intent(in) :: depth
    integer, intent(in), optional :: oneRay ! if we want only one ray from oneRay'th segemtn of the surface
    !local variables
    integer :: ii , ilxloc , lim1, lim2!
    real :: lxloc, & ! light xlocation
            startpx
    if(present(oneRay)) then
      lim1=oneRay; lim2=oneRay
    else
      lim1=1; lim2=size(normal)
    endif
    do ii=lim1,lim2 
      startpx=(surface(ii)+surface(ii+1))/2
      if(startpx<depth) then
        lxloc=(ii+.5)+(depth-startpx)*normal(ii);
          !write(0,*) startpx,lxloc 
        call doBinning1D(lxloc, 1., slice); 
      endif

    enddo
  end subroutine

  !
  subroutine addSurface() !adds the surface to the final image
  implicit none
    real :: ampmax
    integer:: ii
    ampmax=getArrayMax(data(:,:))!/size(data(1,:));
    do ii=1, size(surface)
      !call doBinning1D(surface(ii),ampmax,data(:,ii))
      if(surface(ii)>1 .and. surface(ii)<size(data(:,1))) then
        data(1:nint(surface(ii)),ii)=ampmax;
      endif
    enddo
  end subroutine
end program
