program model3d
use sep
use waterlib
implicit none
  ! inputs
  real, dimension (:,:), allocatable :: surface, data
  real, dimension (:,:,:,:), pointer :: normal
  integer :: n1, n2, n3, dmin, SurfaceCount, DepthCount
  !local variabels
  integer :: k,g
  call sep_init();
  call from_history("n1", n1);
  call from_history("n2", n2);
  call from_history("n3", SurfaceCount,1);
  write(0,*) "Surface count: ", SurfaceCount
  call from_param("n3", DepthCount,100);
  write(0,*) "Depth count: ", DepthCount
  if(SurfaceCount /=1 .and. DepthCount /=1 ) then
    !Either model several depths under one invarient surface
    !or model under changing surface under a single depth
    call erexit("You can't have both n3 from history and n3 from parameter not equal one.");
  else
    n3 = max(SurfaceCount, DepthCount);
  endif
  call from_param("dmin", dmin, 1);
  call to_history("n3", n3);
  allocate(surface(n1, n2), data(n1,n2));
  do g = 1, SurfaceCount
    call sep_read(surface);
    call makeSurfaceNormal3D(surface, normal)
    write(0,*) 'at surface =',g
    do k = dmin, dmin+DepthCount-1 
      write(0,*) 'at depth =', k
      data=0
      call make3dLightScan(data,real(k));
      call sep_write(data); 
    enddo
  enddo
  deallocate(surface,normal, data);

contains

  subroutine make3dLightScan(slice, depth)
  implicit none
    real, dimension(:,:):: slice
    real :: depth
    !local variables
    real :: dist, normalx, normaly, deltax, deltay, deltaz,expval,sigma,startpz;
    integer :: ii, jj, nn, mm
    sigma=2
    do jj=1, size(data(1,:))!loop over the output domain
        write(0,*) 'at jj=', jj 
      do ii=1,size(data(:,1))   
        if(surface(ii,jj)>depth) then !make sure we are below water surface
          data(ii,jj)=0.0;
        else
          do nn=1, size(normal(:,1,1,1))!, size(normal(:,1,1,1))*3/100
            do mm=1, size(normal(1,:,1,1))!, size(normal(1,:,1,1))*3/100
              !for triangle one
              deltax = ii-nn-CNRML_POS1; 
              deltay = jj-mm-CNRML_POS1;
              startpz = getTrigMidPointZ(  Surface(nn,mm),  &
                                       Surface(nn,mm+1), &
                                       Surface(nn+1,mm));
              deltaz = (depth-startpz);
              normalx = normal(nn,mm,CNRML_TRI1,CNRML_DSDX);
              normaly = normal(nn,mm,CNRML_TRI1,CNRML_DSDY);
              dist    = sqrt( deltax*deltax + &
                         deltay*deltay + &
                         (deltaz*deltaz) );
              expval  =  sqrt((normaly*deltaz-1*deltay)**2. + &
                          (1*deltax-normalx*deltaz)**2.+&
                          (normalx*deltay-normaly*deltax)**2.)/ &
                          (sqrt(normalx**2.+normaly**2.)+dist)
              slice(ii,jj) = slice(ii,jj) + &
                    exp(-dist*expval/sigma)/(2*sigma)* exp(-0.00*dist);
              
              !for triangle two
              deltax=ii-nn-CNRML_POS2; 
              deltay=jj-mm-CNRML_POS2;
              startpz=getTrigMidPointZ(Surface(nn+1,mm+1),&
                                       Surface(nn,mm+1), &
                                       Surface(nn+1,mm));
              deltaz=(depth-startpz);
              normalx=normal(nn,mm,CNRML_TRI2,CNRML_DSDX);
              normaly=normal(nn,mm,CNRML_TRI2,CNRML_DSDY);
              dist=sqrt( deltax*deltax + &
                         deltay*deltay + &
                         (deltaz*deltaz) );
              expval  =  sqrt((normaly*deltaz-1*deltay)**2. + &
                          (1*deltax-normalx*deltaz)**2.+&
                          (normalx*deltay-normaly*deltax)**2.)/ &
                          (sqrt(normalx**2.+normaly**2.)+dist)
              slice(ii,jj) = slice(ii,jj) + &
                    exp(-dist*expval/sigma)/(2*sigma)* exp(-0.00*dist);
            enddo
          enddo
        endif
      enddo
    enddo
  end subroutine
end program
