module water2dlib 
use sep 
  real , parameter:: PI=3.14 !acos(-1.);
contains
  !builds a 2D surface
  function model2d(surface, depth, precomp_normal)
  implicit none
    real, dimension(:) :: surface
    !for precomputed normal
    real, optional, dimension(:), target:: precomp_normal 
    real, intent(in) :: depth
    !local variables
    real, dimension(size(surface)):: model2d
    real, dimension (:), pointer ::  normal
    if(present(precomp_normal)) then
      normal=>precomp_normal; 
      !if the surface do not change. i.e. when we model several depths for the same surface
    else
      normal=>getSurfaceNormal(surface);
    endif
    model2d=0;
    call makeLightScan(surface, normal, model2d, depth);
    nullify(normal);
  end function

  function getSurfaceNormal(surface)
    real, dimension(:):: surface 
    real, dimension(:), pointer :: getSurfaceNormal 
    integer :: ii
    allocate(getSurfaceNormal(size(surface-1)));
    getSurfaceNormal=0;
    do ii=1, size(getSurfaceNormal)
      getSurfaceNormal(ii) = -(surface(ii+1)-surface(ii));
    enddo
  end function
  !the two dimensional version
  subroutine makeLightScan(surface, normal, slice, depth, oneRay) 
  implicit none
    real, intent(inout), dimension(:) :: slice
    real, intent(in), dimension(:) :: surface, normal 
    real, intent(in) :: depth
    integer, intent(in), optional :: oneRay ! if we want only one ray from oneRay'th segemtn of the surface
    !local variables
    integer :: ii, jj , ilxloc , lim1, lim2!
    real :: lxloc, & ! light xlocation
            startpx, startpy, endpx, endpy, slope, dist,&
            sigma, expval,deltax, deltay
    !slice=0;
    if(present(oneRay)) then
      lim1=oneRay; lim2=oneRay
    else
      lim1=1; lim2=size(normal)
    endif
    do ii=1, size(slice)
      if(surface(ii)<depth) then ! if depth is above me why bother
        endpx=ii; endpy=depth
        do jj=lim1,lim2 
          startpy=(surface(jj)+surface(jj+1))/2.
          startpx=jj+.5
            !sigma=1.5;! for constant width Gbeam
            !sigma=.02; ! for diffusing Gbeam
            !sigma=.15; ! for contstant-diffusing Gbeam
            !sigma=.1; ! for diffusing Lbeam
            sigma=6; ! for const width Lbeam
            deltax=endpx-startpx;
            deltay=endpy-startpy;
            dist=sqrt(deltax**2.+deltay**2.);
            expval=abs(normal(jj)*deltay-deltax)/ &
            (sqrt(normal(jj)**2.+1)*dist);
            slice(ii)=slice(ii)+ &
                !exp(-dist**2.*(expval)**2./(2*sigma**2.))/(sigma*sqrt(2*PI))*exp(-0*0.005*dist); ! constant-Gaussian
                !exp(-(expval)**2./(2*sigma**2.))/(sigma*sqrt(2*PI))*exp(-0.005*dist); ! diffusing Gaussian
                !exp(-dist*(expval)**2./(2*sigma**2.))/(sigma*sqrt(2*PI))*exp(-0.05*dist); ! const-difuse Gaussian
                !exp(-abs(expval)/(sigma))/(2*sigma)*exp(-0.005*dist);
                exp(-dist*abs(expval)/(sigma))/(2*sigma)*exp(-0.001*dist);!Const width LBeam
        enddo
      endif
    enddo
  end subroutine
end module
