module waterlib
  !indexes to deal with Trig-mesh normals
  integer, parameter :: CNRML_DSDX=1, CNRML_DSDY=2, &
                        CNRML_TRI1=1, CNRML_TRI2=2;
  real, parameter    :: CNRML_SN = 1-.707, &!sqrt(.5), &
                        CNRML_POS1 = CNRML_SN, & 
                        ! or 1./(sqrt(2.)+2.), &
                        CNRML_POS2=1-CNRML_SN
contains
  !bins a value to its nearst intiger between the upper and lower bounds
  subroutine doBinning1D(x, value, arry)
    real, intent(in):: x, value
    real, intent(inout), dimension(:) :: arry
    bin=nint(x)
    if(bin>0 .and. bin<size(arry)-1) then 
        arry(bin)=arry(bin)+value; !value*(1-(x-bin))
        !arry(bin+1)=arry(bin)+value*((x-bin))
    endif
  end subroutine
  
  real function getArrayMax(ary)
    real, dimension(:,:) :: ary
    integer :: ii,jj
    getArrayMax=ary(1,1);
    do jj=1, size(ary(0,:))
      do ii=1, size(ary(:,0))
        if(getArrayMax<ary(ii,jj)) getArrayMax=ary(ii,jj)
      enddo
    enddo
  end function

  real function getArrayAverage(ary)
  implicit none
    real, dimension(:,:) :: ary
    real:: sumall
    integer :: ii
    sumall=0;
    getArrayAverage=sum(ary)/size(ary)
  end function
  
  ! Obseleat see next
  subroutine DONT_USE_makeSurfaceNormal3D_OLD(surface, normal)
    real, dimension(:,:) :: surface
    real, intent(out), dimension(:,:,:), pointer :: normal
    integer:: ii, jj
    allocate(normal(size(surface(:,1))-1,size(surface(1,:))-1,2));
    do ii=1, size(normal(:,1,1))
      do jj=1, size(normal(1,:,1))
        normal(ii,jj,1)=surface(ii+1,jj)-surface(ii,jj);
        normal(ii,jj,2)=surface(ii,jj+1)-surface(ii,jj);
      enddo
    enddo
    
  end subroutine
  
  subroutine makeSurfaceNormal3D(surface, normal)
    real, dimension(:,:) :: surface
    real, intent(out), dimension(:,:,:,:), pointer :: normal
    integer:: ii, jj
    allocate(normal(size(surface(:,1))-1,size(surface(1,:))-1,2,2));
    do ii=1, size(normal(:,1,1,1))
      do jj=1, size(normal(1,:,1,1))
        normal(ii,jj,CNRML_TRI1,CNRML_DSDX)=  &
                    -(surface(ii+1,jj)-surface(ii,jj));
        normal(ii,jj,CNRML_TRI1,CNRML_DSDY)=  &
                    -(surface(ii,jj+1)-surface(ii,jj));
        normal(ii,jj,CNRML_TRI2,CNRML_DSDX)=  &
                    -(surface(ii+1,jj+1)-surface(ii,jj+1));
        normal(ii,jj,CNRML_TRI2,CNRML_DSDY)=  &
                    -(surface(ii+1,jj+1)-surface(ii+1,jj));
      enddo
    enddo
  end subroutine

  real function getTrigMidPointZ(az, bz, cz)
  !   a--1--b
  !   | p  /
  !   1   /
  !   |  /
  !   | /
  !   c
  !  try to find value at p which is at coordinates (x,y) = (D,D)
  !  where D=(1/(sqrt(2)+2) this points to the center of the triangle
     real, intent(in) :: az, bz, cz
     getTrigMidPointZ = az + CNRML_SN * (bz+cz-2*az)
  end function

end module
